@@ -8,16 +8,13 @@ from .points import BagOfPoints, BagOfVectors, Point, Direction, Plane3D
 from .points import PointCloud, NormalCloud, ImageCoords, RgbCloud, RgbPointCloud, PointNormalCloud
 from .primitives import Box, Contour
 from .rigid_transformations import RigidTransform, SimilarityTransform
-from .utils import gen_experiment_id, histogram, skew, deskew, pretty_str_time, filenames, sph2cart, cart2sph
+from .utils import gen_experiment_id, histogram, skew, deskew, pretty_str_time, filenames, sph2cart, cart2sph, is_positive_definite, is_positive_semi_definite, keyboard_input
 from .yaml_config import YamlConfig
 from .dist_metrics import abs_angle_diff, DistMetrics
-from .random_variables import RandomVariable, BernoulliRV, GaussianRV, ArtificialRV, ArtificialSingleRV, IsotropicGaussianRigidTransformRandomVariable
+from .random_variables import RandomVariable, BernoulliRV, GaussianRV, ArtificialRV, ArtificialSingleRV, GaussianRigidTransformRandomVariable, IsotropicGaussianRigidTransformRandomVariable
 from .completer import Completer
 from .learning_analysis import ConfusionMatrix, ClassificationResult, BinaryClassificationResult, RegressionResult
 from .tensor_dataset import Tensor, TensorDatapoint, TensorDataset
-
-try:
-    from .data_stream_syncer import DataStreamSyncer
-    from .data_stream_recorder import DataStreamRecorder
-except Exception:
-    print("Unable to import DataStreamSyncer and Recorder! Likely due to missing multiprocess")
+from .logger import Logger
+from .data_stream_syncer import DataStreamSyncer
+from .data_stream_recorder import DataStreamRecorder
