@@ -353,6 +353,10 @@ class BinaryClassificationResult(ClassificationResult):
         self.threshold = threshold
         ClassificationResult.__init__(self, pred_probs, labels)
 
+    @property
+    def cross_entropy_loss(self):
+        return np.mean((-1 * self.labels * np.log(self.pred_probs)) + (-1 * (1 - self.labels) * np.log(1 - self.pred_probs)))
+
     @property
     def num_categories(self):
         return 2
@@ -447,6 +451,13 @@ class BinaryClassificationResult(ClassificationResult):
     def pct_pred_neg(self):
         return 1.0 - self.pct_pred_pos
 
+    @property
+    def cross_entropy_loss(self):
+        probs = self.pred_probs.copy()
+        probs[probs == 1.0] = 0.99999
+        probs[probs == 0.0] = 0.00001
+        return -np.mean(self.labels * np.log(probs) + (1.0 - self.labels) * np.log(1.0 - probs))
+    
     @property
     def sorted_values(self):
         # sort by prob
