@@ -6,6 +6,7 @@ import logging
 import os
 
 import numpy as np
+import scipy.linalg
 
 from . import utils
 from . import transformations
@@ -74,8 +75,8 @@ class RigidTransform(object):
 
         self.rotation = rotation
         self.translation = translation
-        self._from_frame = from_frame
-        self._to_frame = to_frame
+        self._from_frame = str(from_frame)
+        self._to_frame = str(to_frame)
 
     def copy(self):
         """Returns a copy of the RigidTransform.
@@ -174,7 +175,7 @@ class RigidTransform(object):
 
     @from_frame.setter
     def from_frame(self, from_frame):
-        self._from_frame = from_frame
+        self._from_frame = str(from_frame)
 
     @property
     def to_frame(self):
@@ -184,7 +185,7 @@ class RigidTransform(object):
 
     @to_frame.setter
     def to_frame(self, to_frame):
-        self._to_frame = to_frame
+        self._to_frame = str(to_frame)
 
     @property
     def euler_angles(self):
@@ -211,6 +212,20 @@ class RigidTransform(object):
         qd = np.append([0], self.translation / 2.)
         return DualQuaternion(qr, qd)
 
+    @property
+    def axis_angle(self):
+        """:obj:`numpy.ndarray` of float: The axis-angle representation for the rotation.
+        """
+        qw, qx, qy, qz = self.quaternion
+        theta = 2 * np.arccos(qw)
+        omega = np.array([1,0,0])
+        if theta > 0:
+            rx = qx / np.sqrt(1.0 - qw**2)
+            ry = qy / np.sqrt(1.0 - qw**2)
+            rz = qz / np.sqrt(1.0 - qw**2)
+            omega = np.array([rx, ry, rz])
+        return theta * omega
+    
     @property
     def euler(self):
         """TODO DEPRECATE THIS?"""
@@ -437,7 +452,7 @@ class RigidTransform(object):
         if isinstance(rigid_object, BagOfPoints):
             return self.apply(rigid_object)
         raise ValueError('Cannot multiply rigid transform with object of type %s' %(type(rigid_object)))
-
+                              
     def inverse(self):
         """Take the inverse of the rigid transform.
 
@@ -446,9 +461,9 @@ class RigidTransform(object):
         :obj:`RigidTransform`
             The inverse of this RigidTransform.
         """
-        inv_pose = np.linalg.inv(self.matrix)
-        rotation, translation = RigidTransform.rotation_and_translation_from_matrix(inv_pose)
-        return RigidTransform(rotation, translation,
+        inv_rotation = self.rotation.T
+        inv_translation = np.dot(-self.rotation.T, self.translation)
+        return RigidTransform(inv_rotation, inv_translation,
                               from_frame=self._to_frame,
                               to_frame=self._from_frame)
 
@@ -580,10 +595,46 @@ class RigidTransform(object):
         return out
 
     def __repr__(self):
-        out = 'RigidTransform(rotation={0}, translation={1}, from_frame={2}, to_frame={3})'.format(self.rotation,
-                self.translation, self.from_frame, self.to_frame)
+        out = 'RigidTransform(rotation=np.{0}, translation=np.{1}, from_frame={2}, to_frame={3})'.format(repr(self.rotation),
+                repr(self.translation), repr(self.from_frame), repr(self.to_frame))
         return out
 
+    @staticmethod
+    def ros_q_to_core_q(q_ros):
+        """Converts a ROS quaternion vector to an autolab_core quaternion vector."""
+        q_core = np.array([q_ros[3], q_ros[0], q_ros[1], q_ros[2]])
+        return q_core
+
+    @staticmethod
+    def core_q_to_ros_q(q_core):
+        """Converts a ROS quaternion vector to an autolab_core quaternion vector."""
+        q_ros = np.array([q_core[1], q_core[2], q_core[3], q_core[0]])
+        return q_ros
+
+    @staticmethod
+    def from_ros_pose_msg(pose_msg,
+                          from_frame='unassigned',
+                          to_frame='world'):
+        """Creates a RigidTransform from a ROS pose msg. 
+        
+        Parameters
+        ----------
+        pose_msg : :obj:`geometry_msgs.msg.Pose`
+            ROS pose message
+        """
+        quaternion = np.array([pose_msg.orientation.w,
+                               pose_msg.orientation.x,
+                               pose_msg.orientation.y,
+                               pose_msg.orientation.z])
+        position = np.array([pose_msg.position.x,
+                             pose_msg.position.y,
+                             pose_msg.position.z])
+        pose = RigidTransform(rotation=quaternion,
+                              translation=position,
+                              from_frame=from_frame,
+                              to_frame=to_frame)
+        return pose        
+        
     @staticmethod
     def from_vec(vec, from_frame='unassigned', to_frame='world'):
         return RigidTransform(rotation=vec[3:],
@@ -591,6 +642,21 @@ class RigidTransform(object):
                               from_frame=from_frame,
                               to_frame=to_frame)
 
+    @staticmethod
+    def from_pose_msg(pose_msg, from_frame='unassigned', to_frame='world'):
+        translation = np.array([pose_msg.position.x,
+                                pose_msg.position.y,
+                                pose_msg.position.z])
+        rotation = np.array([pose_msg.orientation.w,
+                             pose_msg.orientation.x,
+                             pose_msg.orientation.y,
+                             pose_msg.orientation.z])
+        return RigidTransform(rotation=rotation,
+                              translation=translation,
+                              from_frame=from_frame,
+                              to_frame=to_frame)
+
+    
     @staticmethod
     def rigid_transform_from_ros(from_frame, to_frame, service_name='rigid_transforms/rigid_transform_listener', namespace=None):
         """Gets transform from ROS as a rigid transform
@@ -648,6 +714,48 @@ class RigidTransform(object):
         R = transformations.quaternion_matrix(q_xyzw)[:3,:3]
         return R
 
+
+    @staticmethod
+    def quaternion_from_axis_angle(v):
+        """Convert axis-angle representation to a quaternion vector.
+
+        Parameters
+        ----------
+        v : :obj:`numpy.ndarray` of float
+            An axis-angle representation.
+
+        Returns
+        -------
+        :obj:`numpy.ndarray` of float
+            A quaternion vector from the axis-angle vector.
+        """
+        theta = np.linalg.norm(v)
+        if theta > 0:
+            v = v / np.linalg.norm(v)
+        ax, ay, az = v    
+        qx = ax * np.sin(0.5 * theta)
+        qy = ay * np.sin(0.5 * theta)
+        qz = az * np.sin(0.5 * theta)        
+        qw = np.cos(0.5 * theta)
+        q = np.array([qw, qx, qy, qz])
+        return q
+        
+    @staticmethod
+    def rotation_from_axis_angle(v):
+        """Convert axis-angle representation to rotation matrix.
+
+        Parameters
+        ----------
+        v : :obj:`numpy.ndarray` of float
+            An axis-angle representation.
+
+        Returns
+        -------
+        :obj:`numpy.ndarray` of float
+            A 3x3 rotation matrix made from the axis-angle vector.
+        """
+        return RigidTransform.rotation_from_quaternion(RigidTransform.quaternion_from_axis_angle(v))
+        
     @staticmethod
     def transform_from_dual_quaternion(dq, from_frame='unassigned', to_frame='world'):
         """Create a RigidTransform from a DualQuaternion.
@@ -700,6 +808,37 @@ class RigidTransform(object):
         translation = matrix[:3,3]
         return rotation, translation
 
+    @staticmethod
+    def rotation_from_axis_and_origin(axis, origin, angle, to_frame='world'):
+        """
+        Returns a rotation matrix around some arbitrary axis, about the point origin, using Rodrigues Formula
+
+        Parameters
+        ----------
+        axis : :obj:`numpy.ndarray` of float
+            3x1 vector representing which axis we should be rotating about
+        origin : :obj:`numpy.ndarray` of float
+            3x1 vector representing where the rotation should be centered around
+        angle : float
+            how much to rotate (in radians)
+        to_frame : :obj:`str`
+            A name for the frame of reference to which this transform
+            moves objects.
+        """
+        axis_hat = np.array([[0, -axis[2], axis[1]],
+                             [axis[2], 0, -axis[0]],
+                             [-axis[1], axis[0], 0]])
+        # Rodrigues Formula
+        R = RigidTransform(
+            np.eye(3) + np.sin(angle) * axis_hat + (1 - np.cos(angle)) * axis_hat.dot(axis_hat),
+            from_frame=to_frame,
+            to_frame=to_frame
+        )
+
+        return RigidTransform(translation=origin, from_frame=to_frame, to_frame=to_frame) \
+            .dot(R) \
+            .dot(RigidTransform(translation=-origin, from_frame=to_frame, to_frame=to_frame))
+
     @staticmethod
     def x_axis_rotation(theta):
         """Generates a 3x3 rotation matrix for a rotation of angle
@@ -1207,6 +1346,6 @@ class SimilarityTransform(RigidTransform):
         return out
 
     def __repr__(self):
-        out = "SimilarityTransform(rotation={0}, translation={1}, scale={2}, from_frame={3}, to_frame={4})".format(
+        out = "SimilarityTransform(rotation=np.{0}, translation=np.{1}, scale={2}, from_frame={3}, to_frame={4})".format(
                                     repr(self.rotation), repr(self.translation), repr(self.scale), repr(self.from_frame), repr(self.to_frame))
         return out
