@@ -222,13 +222,14 @@ class TensorDataset(object):
     Thus, reads are most efficient when performed in order rather than randomly, to prevent
     expensive I/O to read a single datapoint.
     """
-    def __init__(self, filename, config, access_mode=WRITE_ACCESS):
+    def __init__(self, filename, config, access_mode=WRITE_ACCESS, force_overwrite=False):
         # read params
         self._filename = filename
         self._config = config
         self._metadata = {}
         self._datapoints_per_file = config['datapoints_per_file']
         self._access_mode = access_mode
+        self._filename_numeric_label_place = 5
 
         # open dataset folder
         # create dataset if necessary
@@ -239,10 +240,11 @@ class TensorDataset(object):
             raise ValueError('Dataset %s does not exist!' %(self._filename))
         # check dataset empty
         elif access_mode == WRITE_ACCESS and os.path.exists(self._filename) and len(os.listdir(self._filename)) > 0:
-            human_input = keyboard_input('Dataset %s exists. Overwrite?' %(self._filename), yesno=True)
-            if human_input.lower() == 'n':
-                raise ValueError('User opted not to overwrite dataset')
-
+            if not force_overwrite:
+                human_input = keyboard_input('Dataset %s exists. Overwrite?' %(self._filename), yesno=True)
+                if human_input.lower() == 'n':
+                    raise ValueError('User opted not to overwrite dataset')
+                
             # delete the old dataset
             shutil.rmtree(self._filename)
             os.mkdir(self._filename)
@@ -282,29 +284,45 @@ class TensorDataset(object):
             if os.path.exists(self.metadata_filename):
                 self._metadata = json.load(open(self.metadata_filename, 'r'))
             
+            # get the filename numeric label place
+            sample_fname = os.listdir(self.tensor_dir)[0]
+            self._filename_numeric_label_place = len(sample_fname[sample_fname.rindex('_') + 1:-4])
+
             # read the number of tensor files
             tensor_dir = self.tensor_dir
             tensor_filenames = filenames(tensor_dir, tag=COMPRESSED_TENSOR_EXT, sorted=True)
             pruned_tensor_filenames = []
             for filename in tensor_filenames:
                 try:
-                    file_num = int(filename[-9:-4])
+                    file_num = int(filename[-4 - self._filename_numeric_label_place:-4])
                     pruned_tensor_filenames.append(filename)
                 except:
                     pass
             tensor_filenames = pruned_tensor_filenames    
-            file_nums = np.array([int(filename[-9:-4]) for filename in tensor_filenames])
+            file_nums = np.array([int(filename[-4 - self._filename_numeric_label_place:-4]) for filename in tensor_filenames])
             if len(file_nums) > 0:
                 self._num_tensors = np.max(file_nums)+1
             else:
                 self._num_tensors = 0
 
             # compute the number of datapoints
-            last_tensor_ind = np.where(file_nums == self._num_tensors-1)[0][0]
-            last_tensor_data = np.load(tensor_filenames[last_tensor_ind])['arr_0']
-            self._num_datapoints_last_file = last_tensor_data.shape[0]
-            self._num_datapoints = self._datapoints_per_file * (self._num_tensors-1) + self._num_datapoints_last_file
-
+            self._num_datapoints_last_file = 0
+            self._num_datapoints = 0
+            if file_nums.shape[0] > 0:
+                last_tensor_ind = np.where(file_nums == self._num_tensors-1)[0][0]
+                found_last_file = False
+                while self._num_tensors >= 0 and not found_last_file:
+                    try:
+                        last_tensor_data = np.load(tensor_filenames[last_tensor_ind])['arr_0']
+                        found_last_file = True
+                    except:
+                        found_last_file = False
+                        self._num_tensors -= 1
+                        last_tensor_ind = np.where(file_nums == self._num_tensors-1)[0][0]
+                        
+                self._num_datapoints_last_file = last_tensor_data.shape[0]
+                self._num_datapoints = self._datapoints_per_file * (self._num_tensors-1) + self._num_datapoints_last_file
+                
             # set file index
             cur_file_num = 0
             start_datapoint_index = 0
@@ -327,6 +345,10 @@ class TensorDataset(object):
                 # set mapping from index to file num
                 self._index_to_file_num[ind] = cur_file_num
 
+    @property
+    def filename_numeric_label_place(self):
+        return self._filename_numeric_label_place
+
     @property
     def filename(self):
         return self._filename
@@ -391,6 +413,13 @@ class TensorDataset(object):
     def split_dir(self):
         """ Return the tensor directory. """
         return os.path.join(self._filename, 'splits')
+
+    @property
+    def split_names(self):
+        """ Returns the list of all splits
+        """
+        split_names = os.listdir(self.split_dir)
+        return split_names
     
     def datapoint_indices_for_tensor(self, tensor_index):
         """ Returns the indices for all datapoints in the given tensor. """
@@ -409,7 +438,7 @@ class TensorDataset(object):
         file_ext = TENSOR_EXT
         if compressed:
             file_ext = COMPRESSED_TENSOR_EXT
-        filename = os.path.join(self.filename, 'tensors', '%s_%05d%s' %(field_name, file_num, file_ext))
+        filename = os.path.join(self.filename, 'tensors', ('%s_%0' + str(self._filename_numeric_label_place) + 'd%s') %(field_name, file_num, file_ext))
         return filename
 
     def train_indices_filename(self, split_name):
@@ -741,7 +770,7 @@ class TensorDataset(object):
         val_indices = np.load(val_filename)['arr_0']
         return train_indices, val_indices, metadata
         
-    def make_split(self, split_name, train_pct=0.8, field_name=None):
+    def make_split(self, split_name, val_indices=None, train_pct=0.8, field_name=None):
         """ Splits the dataset into train and test according
         to the given attribute.
         The split is saved with the dataset for future access.
@@ -750,6 +779,8 @@ class TensorDataset(object):
         ----------
         split_name : str
             name of the split (for future accesses)
+        val_indices : 
+            indices of the validation datapoints (overrides other variables if not None) 
         train_pct : float
             percent of data to use for training
         field_name : str
@@ -769,9 +800,12 @@ class TensorDataset(object):
         # check existence
         if self.has_split(split_name):
             raise ValueError('Cannot create split %s - it already exists! To overwrite, delete split with TensorDataset.delete_split(split_name)' %(split_name))
-        
+
         # perform splitting
-        if field_name is None:
+        if val_indices is not None:
+            all_indices = np.arange(self.num_datapoints)
+            train_indices = np.setdiff1d(all_indices, val_indices)
+        elif field_name is None:
             # split on indices
             indices = np.arange(self.num_datapoints)
             num_train = int(train_pct * self.num_datapoints)
